@@ -6,26 +6,24 @@ They could do with some sorting and refactoring.
 import contextlib
 import json
 import os
+import random
+import re
+import sqlite3
+import string
 import warnings
 from functools import partial
+from pathlib import Path
 
 import numpy as np
-from besos.IDF_class import IDF, view_epJSON
+import pandas as pd
+from eppy.bunch_subclass import BadEPFieldError
 from shapely.geometry import LineString, Point, Polygon
 
 import config
 import eplus_funcs as eplus
+from besos.IDF_class import IDF, view_epJSON
 from errors import ModeError
 
-import sqlite3
-import pandas as pd
-import csv
-import re
-from pathlib import Path
-
-import random
-import string
-
 
 def view_building(building):
     if get_mode(building) == "json":
@@ -58,7 +56,7 @@ def convert_format(s: str, place, mode):
 
     :param s: name to convert
     :param place: the type of place that is being used 'field' or 'class'
-    :param mode: whether to convert to idf or json formating
+    :param mode: whether to convert to idf or json formatting
     :return: the converted name
     """
     # TODO: Find an authoritative source for the naming conventions
@@ -92,18 +90,29 @@ def get_idf(
     ep_path=None,
 ) -> IDF:
     """Uses eppy to read an idf file and generate the corresponding idf object"""
+
     if idd_file is None:
-        ep_exe, ep_directory = eplus.get_ep_path(version, ep_path)
-        idd_file = os.path.join(ep_directory, "Energy+.idd")
-    # Trying to change the idd file inside a program will cause an error
-    IDF.setiddname(idd_file)
+        # set iddname to default value if needed.
+        _, ep_directory = eplus.get_ep_path(version, ep_path)
+        default_idd = Path(ep_directory, "Energy+.idd").resolve()
+        if IDF.iddname is None:
+            IDF.setiddname(str(default_idd))
+        elif Path(IDF.iddname).resolve() != default_idd:
+            warnings.warn(
+                f"idd is already set to: {IDF.iddname}. "
+                f"It will NOT be changed to the default of: {default_idd}."
+            )
+    else:
+        # Trying to change the idd file inside a program will cause an error
+        # paths must be converted to strings before sending to eppy
+        IDF.setiddname(str(idd_file))
     # TODO: Fix this rather than hiding it.
     # calling IDF causes a warning to appear, currently redirect_stdout hides this.
     with contextlib.redirect_stdout(None):
         idf = IDF(idf_file)
-        # override the output location so I stop messing up
-        idf.run = partial(idf.run, output_directory=output_directory)
-        return idf
+    # override the output location so I stop messing up
+    idf.run = partial(idf.run, output_directory=output_directory)
+    return idf
 
 
 def get_building(
@@ -120,10 +129,10 @@ def get_building(
         building = config.files.get("building")
         mode = config.energy_plus_mode
     if mode is None:
-        ending = building.split(".")[-1]
-        if ending == "idf":
+        ending = Path(building).suffix
+        if ending == ".idf":
             mode = "idf"
-        elif ending == "epJSON":
+        elif ending == ".epJSON":
             mode = "json"
         else:
             warnings.warn(
@@ -132,9 +141,6 @@ def get_building(
     if building is None:
         building = config.files[mode]
     if mode == "idf":
-        if data_dict is None:
-            _, ep_directory = eplus.get_ep_path(version, ep_path)
-            data_dict = os.path.join(ep_directory, "Energy+.idd")
         idf = get_idf(
             idf_file=building, idd_file=data_dict, output_directory=output_directory
         )
@@ -145,6 +151,36 @@ def get_building(
     ModeError(mode)
 
 
+def get_idfobject_from_name(idf: IDF, name: str):
+    """	get_idfobject_from_name(IDF, str)
+
+    Gets an object from the passed idf where it's name
+    value is equal to the passed string, if none are found
+    then this method returns None
+
+    Args:
+        idf(IDF): The idf to find the object from.
+        name(str): The string to find that is equal to the
+               name field of the object.
+    """
+
+    #: Scan all objects in all lists
+    for idfobject_list in idf.idfobjects.values():
+        for obj in idfobject_list:
+            #: Try except block incase object does not
+            #: have a .Name attribute
+            try:
+                # If the object has the name we are looking for return it.
+                if obj.Name == name:
+                    return obj
+            # ignore objects with no Name
+            # Note that the BadEPFieldError may not be possible
+            except (BadEPFieldError,):
+                pass
+    #: If this is encountered then no match was found, so we return None.
+    return None
+
+
 def get_windows(building):
     mode = get_mode(building)
     if mode == "idf":
@@ -164,7 +200,7 @@ def wwr_all(building, wwr: float, direction=None) -> None:
     """
     mode = get_mode(building)
     windows = get_windows(building)
-    if direction == None:
+    if direction is None:
         for window_name, window in windows:
             wwr_single(window, wwr, building, mode)
     else:
@@ -205,15 +241,11 @@ def get_coordinates(window, building, mode):
         def coordinates(ax):
             return [window[f"Vertex_{n}_{ax.upper()}coordinate"] for n in range(1, 5)]
 
-        wall = building.getobject(
-            "BUILDINGSURFACE:DETAILED", window.Building_Surface_Name
-        )
     elif mode == "json":
 
         def coordinates(ax):
             return [window[f"vertex_{n}_{ax.lower()}_coordinate"] for n in range(1, 5)]
 
-        wall = building["BuildingSurface:Detailed"][window["building_surface_name"]]
     else:
         raise ModeError(mode)
     coordinate = {}
@@ -234,7 +266,7 @@ def one_window(building):
     """Removes some windows so that each wall has no more than one"""
     mode = get_mode(building)
     walls = set()
-    toRemove = []
+    to_remove = []
     windows = get_windows(building)
     for window_name, window in windows:
         if mode == "idf":
@@ -244,14 +276,14 @@ def one_window(building):
         else:
             raise ModeError(mode)
         if wall_name in walls:
-            toRemove.append((window_name, window))
+            to_remove.append((window_name, window))
         else:
             walls.add(wall_name)
     if mode == "idf":
-        for window_name, window in toRemove:
+        for window_name, window in to_remove:
             building.idfobjects["FENESTRATIONSURFACE:DETAILED"].remove(window)
     elif mode == "json":
-        for window_name, window in toRemove:
+        for window_name, window in to_remove:
             building["FenestrationSurface:Detailed"].pop(window_name)
 
 
@@ -275,55 +307,47 @@ def wwr_single(window, wwr: float, building, mode, direction=None, values=None):
     else:
         raise ValueError("The window is not aligned with the x or y axes")
     # with direction specified or not
-    if direction != None:
-        if check_direct(direction, xs, ys, values) == True:
+    if direction is not None:
+        if check_direct(direction, xs, ys, values):
             set_wwr_single(window, wwr, axs, zs, axis)
     else:
         set_wwr_single(window, wwr, axs, zs, axis)
 
 
-def check_direct(direction, xs, ys, values):
-    """ check if the window's direction is matched with the desired direction
-    
-        :param values: the dictionary of max and min range of all windows
-        :param 
-        :return: True or False
+# TODO document xs and ys
+def check_direct(direction: str, xs, ys, values) -> bool:
+    """check if the window's direction is the same as the desired direction
+
+    :param direction: the direction to compare with
+    :param values: the dictionary of max and min range of all windows
+    :return:
     """
     direction = direction.upper()
-    if (
-        direction != "NORTH"
-        and direction != "WEST"
-        and direction != "EAST"
-        and direction != "SOUTH"
-    ):
+    if direction not in {"NORTH", "WEST", "EAST", "SOUTH"}:
         raise NameError("Direction should be either north, east, west, or south")
     if type(direction) != str:
         raise TypeError("Direction should be a string")
     direct = ""
     if max(ys) == min(ys):
-        axis = "x"
         if max(ys) == values["max_y"]:
             direct = "NORTH"
         else:
             direct = "SOUTH"
     elif max(xs) == min(xs):
-        axis = "y"
-        axs = "WE"
         if max(xs) == values["max_x"]:
             direct = "EAST"
         else:
             direct = "WEST"
-    if direct == direction:
-        return True
-    else:
-        return False
+    return direct == direction
 
 
-def set_wwr_single(window, wwr, axs, zs, axis):
+# TODO: Document axis
+def set_wwr_single(window, wwr: float, axs, zs, axis):
     """ Set the single window's wwr
 
-        :param window: the window need to be modified
-        :param axs: the axis this window is aligned to
+        :param window: the window to be modified
+        :param wwr: the window to wall ratio of the result
+        :param axs: the axis this window is aligned with
         :param zs: the z coordinate of the window
     """
     width = max(axs) - min(axs)
@@ -332,10 +356,10 @@ def set_wwr_single(window, wwr, axs, zs, axis):
     height = max(zs) - min(zs)
     new_height = height * scale_factor
 
-    startW = (width - new_width) / 2
-    endW = startW + new_width
-    startH = (height - new_height) / 2
-    endH = startH + new_height
+    start_width = (width - new_width) / 2
+    end_width = start_width + new_width
+    start_height = (height - new_height) / 2
+    end_height = start_height + new_height
     # Maintains vertex order by mimicking the current order
     s = [0] * 4
     for vertex in range(0, 4):
@@ -344,20 +368,20 @@ def set_wwr_single(window, wwr, axs, zs, axis):
             if axs[vertex] == max(axs):
                 # TOP RIGHT
                 s[0] += 1
-                set_vertex(window, vertex + 1, z=endH, **{axis: endW})
+                set_vertex(window, vertex + 1, z=end_height, **{axis: end_width})
             else:
                 # TOP LEFT
                 s[1] += 1
-                set_vertex(window, vertex + 1, z=endH, **{axis: startW})
+                set_vertex(window, vertex + 1, z=end_height, **{axis: start_width})
         else:
             if axs[vertex] == max(axs):
                 # BOTTOM RIGHT
                 s[2] += 1
-                set_vertex(window, vertex + 1, z=startH, **{axis: endW})
+                set_vertex(window, vertex + 1, z=start_height, **{axis: end_width})
             else:
                 # BOTTOM LEFT
                 s[3] += 1
-                set_vertex(window, vertex + 1, z=startH, **{axis: startW})
+                set_vertex(window, vertex + 1, z=start_height, **{axis: start_width})
     assert s == [1] * 4, ("verticesS are wrong:", s)
 
 
@@ -430,7 +454,7 @@ def get_vertices(building, zone_name, window, distance):
     vertices = [v1, v2, v3]
 
     def subtract(v1, v2):
-        return (v1[0] - v2[0], v1[1] - v2[1], v1[2] - v2[2])
+        return v1[0] - v2[0], v1[1] - v2[1], v1[2] - v2[2]
 
     for i in range(3):
         for j in range(i + 1, 3):
@@ -488,11 +512,14 @@ def get_reference_point(p1, p2, distance):
 
 
 def read_sql(path: str, cmds: list, direction=None):
-    """ Open sql file wiht connect
+    """ Open sql file with connect
 
         :param path: absolute directory to the sql file.
-        :param cmds: list of commands need to be processed, 'all' will process all cmds. All cmds available : 'all', 'wall area', 'ceiling height', 'floor area', 'volume'
-        :param direction: a list of directions of walls when process wall area cmd, None means taking both area and gross area of walls in all directions and the total areas of all walls.
+        :param cmds: list of commands need to be processed, 'all' will process all cmds.
+            All commands available : 'all', 'wall area', 'ceiling height', 'floor area', 'volume'
+        :param direction: a list of directions of walls when process wall area cmd,
+            None means taking both area and gross area of walls in all directions and
+            the total areas of all walls.
         :return: a dictionary of all data desired.
 
     """
@@ -503,7 +530,7 @@ def read_sql(path: str, cmds: list, direction=None):
     except IOError:
         print("Either no such file or file path is not correct.")
 
-    if isinstance(cmds, list) != True:
+    if not isinstance(cmds, list):
         raise TypeError("cmds has be a list")
 
     def get_azim_range(direction: str):
@@ -523,18 +550,23 @@ def read_sql(path: str, cmds: list, direction=None):
             )
         return azim_range
 
-    def wallArea(dire: str):
-        """Find total area with window substracted of either all walls or only walls in current direction
+    def wall_area(dire: str):
+        """Find total area with window subtracted from either all walls or only walls in current direction
         """
         table_name = "Surfaces"
-        azim_range = []
-        query = ""
         azim_range = get_azim_range(dire)
         if len(azim_range) > 2:
-            query = f"SELECT SUM(Area), SUM(GrossArea) FROM {table_name} WHERE (ClassName = 'Wall' AND ((Azimuth >= {azim_range[0]} AND Azimuth <= {azim_range[1]}) or (Azimuth >= {azim_range[2]} AND Azimuth <= {azim_range[3]})))"
+            query = (
+                f"SELECT SUM(Area), SUM(GrossArea) FROM {table_name} WHERE "
+                f"(ClassName = 'Wall' AND ((Azimuth >= {azim_range[0]} AND "
+                f"Azimuth <= {azim_range[1]}) or (Azimuth >= {azim_range[2]} "
+                f"AND Azimuth <= {azim_range[3]})))"
+            )
         else:
-            query = "SELECT SUM(Area), SUM(GrossArea) FROM {} WHERE (ClassName = 'Wall' AND Azimuth >= {} AND Azimuth <= {})".format(
-                table_name, azim_range[0], azim_range[1]
+            query = (
+                f"SELECT SUM(Area), SUM(GrossArea) FROM {table_name} WHERE "
+                f"(ClassName = 'Wall' AND Azimuth >= {azim_range[0]} AND "
+                f"Azimuth <= {azim_range[1]})"
             )
         cur.execute(query)
         data = cur.fetchall()
@@ -544,7 +576,7 @@ def read_sql(path: str, cmds: list, direction=None):
         """Find height of the ceiling
         """
         floor = floor_area()
-        volume = zoneVolume()
+        volume = zone_volume()
         height = volume / floor
         return height
 
@@ -557,8 +589,8 @@ def read_sql(path: str, cmds: list, direction=None):
         data = cur.fetchall()
         return data[0][0]
 
-    def zoneVolume():
-        """Find total voulme of Zones
+    def zone_volume():
+        """Find total volume of Zones
         """
         table_name = "Zones"
         query = f"SELECT SUM(Volume) FROM {table_name}"
@@ -566,7 +598,7 @@ def read_sql(path: str, cmds: list, direction=None):
         data = cur.fetchall()
         return data[0][0]
 
-    def SG_temp():
+    def sg_temp():
         """ find site ground temperature
         """
         table_name = "ReportDataDictionary"
@@ -583,9 +615,8 @@ def read_sql(path: str, cmds: list, direction=None):
     # And there is no protection when using cmd all and others
     def pull_data():
         result = {}
-        direct = []
         # decide which direction we want
-        if direction == None:
+        if direction is None:
             direct = ["total", "south", "north", "west", "east"]
         else:
             direct = direction
@@ -594,11 +625,11 @@ def read_sql(path: str, cmds: list, direction=None):
                 {
                     "ceiling height": ceiling_height(),
                     "floor area": floor_area(),
-                    "volume": zoneVolume(),
+                    "volume": zone_volume(),
                 }
             )
             for dire in direct:
-                area = wallArea(dire)
+                area = wall_area(dire)
                 result.update(
                     {
                         f"{dire} wall area": area[0][0],
@@ -613,7 +644,7 @@ def read_sql(path: str, cmds: list, direction=None):
                     )
                 elif cmd == "wall area":
                     for dire in direct:
-                        area = wallArea(dire)
+                        area = wall_area(dire)
                         result.update(
                             {
                                 f"{dire} wall area": area[0][0],
@@ -625,12 +656,13 @@ def read_sql(path: str, cmds: list, direction=None):
                 elif cmd == "floor area":
                     result.update({"floor area": floor_area()})
                 elif cmd == "volume":
-                    result.update({"volume": zoneVolume()})
+                    result.update({"volume": zone_volume()})
                 elif cmd == "SGtemp":
-                    result.update({"site ground temp": SG_temp()})
+                    result.update({"site ground temp": sg_temp()})
                 else:
                     raise NameError(
-                        "No such command, available cmds are 'all', 'wall area', 'buiding height', 'floor area'"
+                        "No such command, available cmds are 'all', 'wall area', "
+                        "'ceiling height', 'floor area', 'volume', 'SGtemp'"
                     )
         return result
 
@@ -649,21 +681,18 @@ def write_csv(path: str, dest: str, shape=False):
 
         :param path: path to read sql file
         :param dest: destination to put data
-        :param cmds: commands passing to read_sql()
-        :param direction: direction of the desired walls passing to read_sql()
         :param shape: for special use, only works with right file name
     """
-    # Get data from sql file get all datas for now
+    # Get data from sql file. Currently retrieves all the data.
     data = read_sql(path, ["all"],)
 
     # Get number of floors and shape from file name
     filename = Path(dest).name
     match = re.search(r"M.idf", filename)
-    numOfFloor = 0
-    if match == None:
-        numOfFloor = 1
+    if match is None:
+        num_of_floor = 1
     else:
-        numOfFloor = 3
+        num_of_floor = 3
     if shape:
         shape = filename[0].upper()
 
@@ -673,17 +702,17 @@ def write_csv(path: str, dest: str, shape=False):
     south_wall_area = []
     total_rows = len(df["Window to Wall Ratio"])
     for i in range(total_rows):
-        totalArea = data["total wall gross area"] * (1 - df["Window to Wall Ratio"][i])
-        southArea = data["south wall gross area"] * (1 - df["Window to Wall Ratio"][i])
-        wall_area.append(totalArea)
-        south_wall_area.append(southArea)
+        total_area = data["total wall gross area"] * (1 - df["Window to Wall Ratio"][i])
+        south_area = data["south wall gross area"] * (1 - df["Window to Wall Ratio"][i])
+        wall_area.append(total_area)
+        south_wall_area.append(south_area)
 
     # put data back to csv
     df["wall area net"] = wall_area
     df["wall area gross"] = data["total wall gross area"]
     df["south wall area net"] = south_wall_area
     df["south wall area gross"] = data["south wall gross area"]
-    df["number of floors"] = numOfFloor
+    df["number of floors"] = num_of_floor
     df["ceiling height"] = data["ceiling height"]
     df["total floor area"] = data["floor area"]
     df["total volume"] = data["volume"]
@@ -699,45 +728,44 @@ def get_files(directory: str, filetype: str):
         :param filetype: the desired file's types
         :return: a list of file names 
     """
-    fileNames = []
+    file_names = []
     entries = Path(directory)
     for entry in entries.iterdir():
         match = re.search(f".{filetype}$", entry.name)
         if match:
-            fileNames.append(entry.name)
-    if len(fileNames) == 0:
+            file_names.append(entry.name)
+    if len(file_names) == 0:
         raise NameError(
             "No desired file type matched, please check with the file type again."
         )
-    return fileNames
+    return file_names
 
 
 def generate_dir(dest_folder=None):
     """ func use to generate a directory for besos outputs
     """
     folder = "BESOS_Output"
-    if dest_folder != None:
+    if dest_folder is not None:
         folder = dest_folder
     res = "".join(random.choices(string.ascii_uppercase + string.digits, k=20))
-    while os.path.exists(Path(f"{folder}", res)):
+    while os.path.exists(Path(folder, res)):
         res = "".join(random.choices(string.ascii_uppercase + string.digits, k=20))
-    os.makedirs(Path(f"{folder}", res))
-    dir_ = Path(f"{folder}", res)
+    os.makedirs(Path(folder, res))
+    dir_ = Path(folder, res)
     return dir_
 
 
 def generate_batch(
     account: str, time: str, email: str, task_id: int, cpu_per_task=1, mem=1000,
 ):
-    """ fucntion to write a bash file for runing jupyter on computer canada
+    """ function to write a bash file for running jupyter on computer canada
 
         :param account: account used
         :param time: time for bash job
         :param email: user email
-        :param task_id: task id to be setted
+        :param task_id: task id to use
         :param cpu_per_task: number of cpu used to run one task
         :param mem: memory needs in total
-        :param end_date: end date
     """
     # TODO there might be a calculation relationship between mem, cpu and task_id.
     # If possible give some default value to the arguments
@@ -753,13 +781,13 @@ def generate_batch(
         f"#SBATCH --mem={mem}mb\n"
         f"#SBATCH --mail-user={email}\n"
         f"\n"
-        f"#!generate the virtual enviroment\n"
+        f"#!generate the virtual environment\n"
         f"module load python/3.6\n"
         f"source ~/env/bin/activate\n"
-        f"echo 'prog started at:`date`'\n"
+        f"echo 'program started at:`date`'\n"
         f"srun python cluster.py $SLURM_ARRAY_TASK_ID {task_id}\n"
         f"deactivate\n"
-        f"echo 'prog ended at: `date`'\n"
+        f"echo 'program ended at: `date`'\n"
     )
     f.close()
 
