@@ -1,18 +1,17 @@
 """
 This module provides helper functions for interacting with PyEHub.
 """
-import contextlib
-import config
-from functools import partial, reduce
-import numpy as np
 import operator
-import pandas as pd
-import warnings
+from functools import reduce
+from typing import List
 
-from errors import ModeError
+import numpy as np
+import pandas as pd
 import pyehub
 from pyehub.energy_hub.ehub_model import EHubModel
 
+import config
+
 """This file is a collection of functions to interact with PyEHub."""
 
 # TODO: replace back with yaml loader?
@@ -30,7 +29,7 @@ def get_hub():
 
 def get_by_path(root, items):
     """Access a nested object in root by item sequence. Used to navigate PyEHub's not flat dict."""
-    # TODO: inset try statement here incase the parameter doesn't exist to fail gracefuly
+    # TODO: inset try statement here iscase the parameter doesn't exist to fail gracefully
     return reduce(operator.getitem, items, root)
 
 
@@ -54,13 +53,14 @@ def name_getter(dict):
     return namelist
 
 
-def pyehub_parameter_editor(hub, parameters, values: list):
+# TODO: Is this really needed? Can we just use the transformation function?
+def pyehub_parameter_editor(
+    hub, parameters: List["parameters.Parameter"], values: list
+):
     """Changes the __dict__ of the energy hub for the parameters specified
     on initialization with the values given to evaluator."""
-    if len(parameters) > 0:  # There are paremeters to edit
-        for (parameter, value) in zip(parameters, values):
-            # Change the value of the paremeter in the model's dict
-            set_by_path(hub.__dict__, parameter.parameter_path, value)
+    for (parameter, value) in zip(parameters, values):
+        parameter.selector.set(hub, value)
 
 
 def _split_EP_days(hourly_series):
@@ -85,13 +85,14 @@ def ep_to_eh(input, index_size):
 
     input_dict = input.to_dict()
 
-    # Need to have a better joining on index for the pandas dataframe instead of appending (difficulties to be noted with inserting dictionaries into cells)
+    # Need to have a better joining on index for the pandas dataframe instead of appending
+    # (difficulties to be noted with inserting dictionaries into cells)
     columnnames = []
     for j in input_dict:
         columnnames.append(j)
 
-    # 2x for the number of seasons in the splitting will need to have some value stored from the smarter splitting
-    # or make an easy spot to modify a value that propigates through the whole system.
+    # 2x for the number of seasons in the splitting will need to have some value stored from
+    # the smarter splitting or make an easy spot to modify a value that propigates through the whole system.
     # if using the input for the function
     # index_size *=2
     # which should be something that does this
@@ -119,7 +120,8 @@ def ep_to_eh(input, index_size):
 
             temp_df1 = pd.DataFrame(np.array(cold_input), columns=[j])
             temp_df2 = pd.DataFrame(np.array(warm_input), columns=[j])
-            # instead of appending I can edit the value of the specific cell with the input (ah that gets difficult with dictionaries)
+            # instead of appending I can edit the value of the specific cell with the input
+            # (ah that gets difficult with dictionaries)
             # df_input.at[i, j] = [cold_dict]
             # df_input.iloc[i+len(EPdf), df_input.columns.get_loc(j)] = warm_dict
             # could do some sort of join on the
